import tamber

def basic_test():
	print("event/track\n")

	try:
		e = tamber.Event.track(
			user='user_jctzgisbru', 
			item='item_i5gq90scc1', 
			behavior='mention',
			hit=True,
			context=["recommended", "detail-view"]
		)
		print(e)
	except tamber.TamberError as e:
		print(e)

	print("\ndiscover/recommended\n")

	try:
		d = tamber.Discover.recommended(user='user_jctzgisbru',number=100,)
		print(d)
		for rec in d:
			print "item:%s  score:%s\n" % (rec['item'],rec['score'])
	except tamber.TamberError as e:
		print(e)

def partial_test():
	try:
		tamber.Event.batch(
			events=[
				tamber.Event(
					user='user_fwu592pwmo', 
					item='item_u9nlytt3w5', 
					behavior='mention',
					value=1
				),
				tamber.Event(
					user='user_jctzgisbru', 
					item='item_i5gq90scc1', 
					behavior='mention',
					value= 1
				)
			]
		)
	except tamber.TamberError as e:
		print(e)


print("Running Tests...")

tamber.project_key = 'Mu6DUPXdDYe98cv5JIfX'
tamber.engine_key = 'SbWYPBNdARfIDa0IIO9L'

print("Basic Test:\n")

basic_test()

print("Partial Test:\n")

partial_test()

print("\n\n=================\nTesting Complete\n=================\n\n")



from tamber import api_url, api
import json

class TamberObject():
	def __init__(self, *args, **kwargs):
		super(TamberObject, self).__init__(*args, **kwargs)

class TamberJSONEncoder(json.JSONEncoder):
	def default(self, obj):
		if not isinstance(obj, TamberObject):
			return super(TamberJSONEncoder, self).default(obj)
		return obj.__dict__

class APIResource():
	@staticmethod
	def _get_url_path(resource, command):
		return "/%s/%s" % (resource, command)

	@classmethod
	def _url_path(cls, method):
		return cls._get_url_path(cls.name, method)

	@classmethod
	def _flatten_args(cls, keys=None, **params):
		for k in params:
			if isinstance(params[k], dict) or isinstance(params[k], list):
				params[k] = json.dumps(params[k], cls=TamberJSONEncoder)
		if keys:
			reserved = {'api_url', 'project_key', 'engine_key'}
			return dict((k, params[k]) for k in params if k in keys or k in reserved)
		return params

	@classmethod
	def _call_api(cls, method, url, keys=None, **params):
		args = cls._flatten_args(keys, **params)
		return api.call_api(method, url, **args)

# item, user, behavior
class CreateableAPIResource(APIResource):
	@classmethod
	def create(cls, **params):
		url = cls._url_path('create') 
		return cls._call_api('POST', url, **params)

	@classmethod
	def retrieve(cls, **params):
		url = cls._url_path('retrieve') 
		return cls._call_api('GET', url, **params)

# item, user
class UpdatableAPIResource(APIResource):
	@classmethod
	def update(cls, **params):
		url = cls._url_path('update') 
		return cls._call_api('POST', url, **params)

# item
class RemovableAPIResource(APIResource):
	@classmethod
	def remove(cls, **params):
		url = cls._url_path('remove') 
		return cls._call_api('POST', url, **params)

class Event(APIResource, TamberObject):
	name = 'event'
	def __init__(self, user=None, item=None, behavior=None, value=None, hit=False, context=None, created=None):
		self.user = user
		self.item = item
		self.behavior = behavior
		self.value = value
		self.hit = hit
		self.context = context
		self.created = created

	@classmethod
	def track(cls, **params):
		keys = {'user', 'item', 'behavior', 'value', 'hit', 'context', 'created'}
		return cls._call_api('POST', cls._url_path('track'), keys, **params)

	@classmethod
	def retrieve(cls, **params):
		keys = {'user', 'item', 'behavior', 'created_since', 'created_before', 'number'}
		if 'before' in params:
			params['created_before'] = params['before']
		if 'since' in params:
			params['created_since'] = params['since']
		return cls._call_api('POST', cls._url_path('retrieve'), keys, **params)
	
	@classmethod
	def batch(cls, **params):
		keys = {'events'}
		return cls._call_api('POST', cls._url_path('batch'), keys, **params)

class Discover(APIResource):
	name = 'discover'
	@classmethod
	def recommended(cls, **params):
		keys = {'user', 'number', 'page', 'filter', 'test_events', 'get_properties'}
		return cls._call_api('GET', cls._url_path('recommended'), keys, **params)

	@classmethod
	def similar(cls, **params):
		keys = {'item', 'number', 'page', 'filter', 'test_events', 'get_properties'}
		return cls._call_api('GET', cls._url_path('similar'), keys, **params)

	@classmethod
	def recommendedSimilar(cls, **params):
		keys = {'user', 'item', 'number', 'page', 'filter', 'test_events', 'get_properties'}
		return cls._call_api('GET', cls._url_path('recommended_similar'), keys, **params)

	@classmethod
	def popular(cls, **params):
		keys = {'number', 'page', 'filter', 'get_properties'}
		return cls._call_api('GET', cls._url_path('popular'), keys, **params)

	@classmethod
	def hot(cls, **params):
		keys = {'number', 'page', 'filter', 'get_properties'}
		return cls._call_api('GET', cls._url_path('hot'), keys, **params)

class User(CreateableAPIResource, UpdatableAPIResource):
	name = 'user'

	@classmethod
	def search(cls, **params):
		keys = {'filter'}
		return cls._call_api('GET', cls._url_path('search'), keys, **params)

	@classmethod
	def merge(cls, **params):
		keys = {'from', 'to', 'no_create'}
		return cls._call_api('POST', cls._url_path('merge'), keys, **params)

class Item(CreateableAPIResource, UpdatableAPIResource, RemovableAPIResource):
	name = 'item'

class Behavior(CreateableAPIResource):
	name = 'behavior'



import json

class Filter(dict):
	def __init__(self, function=None, *args):
		if function == 'property':
			self[function] = args[0]
		else:
			self[function] = list(args)

	def __str__(self):
		return json.dumps(self)

And = lambda *args: Filter('and', *args)
Or = lambda *args: Filter('or', *args)
Gt = lambda *args: Filter('gt', *args)
Gte = lambda *args: Filter('gte', *args)
Lt = lambda *args: Filter('lt', *args)
Lte = lambda *args: Filter('lte', *args)
Eq = lambda *args: Filter('eq', *args)
Neq = lambda *args: Filter('neq', *args)
Not = lambda arg: Filter('not', arg)
Property = lambda property: Filter('property', property)


import sys
import json
import base64
import tamber

PYTHON_VERSION = sys.version_info[0]

if PYTHON_VERSION == 2:
	from urllib2 import urlopen, Request, HTTPError
	from urllib import urlencode
	b64encode = lambda s: base64.b64encode(s.encode('ascii'))
elif PYTHON_VERSION == 3:
	from urllib.request import urlopen, Request
	from urllib.error import HTTPError
	from urllib.parse import urlencode
	b64encode = lambda s: base64.b64encode(s.encode('ascii')).decode('ascii')

def call_api(method, url, api_url=None, project_key=None, engine_key=None, **params):
	if api_url is None:
		api_url = tamber.get_api_url()
	if project_key is None:
		project_key = tamber.get_project_key()
	if engine_key is None:
		engine_key = tamber.get_engine_key()
	url = api_url + url
	encoded_params = urlencode(params)
	data = None
	headers = dict()

	if method == 'POST':
		headers['Content-Type'] = 'application/x-www-form-urlencoded'
		data = encoded_params
		if PYTHON_VERSION == 3:
			data = data.encode("utf-8")
	elif method == 'GET':
		url += '?' + encoded_params

	api_version = tamber.get_api_version()
	if api_version != None:
		headers['Tamber-Version'] = api_version
	headers['User-Agent'] = 'Tamber/v1 PythonBindings/%s' % tamber.version_str()
	headers['Authorization'] = 'Basic %s' % b64encode(project_key+':'+engine_key)

	req = Request(url, data=data, headers=headers)

	try:
		resp = urlopen(req)
	except HTTPError as e:
		resp = e
	if PYTHON_VERSION < 3:
		data = json.load(resp)
	else:
		respDecoded = resp.read().decode('utf-8')
		try:
			data = json.loads(respDecoded)
		except Exception as e:
			print(data)
			raise e

	if data['success'] is False:
		raise tamber.error.TamberError(data['error'])
	return data['result']
